/**
 * StreitgenossenBaumbachAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.baumbach;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainer2GUI;
import eu.gronos.kostenrechner.interfaces.StreitgenossenTabellenBehandelnd;
import eu.gronos.kostenrechner.logic.baumbach.BaumbachFormel;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Die Action prüft die Werte auf dem {@link StreitgenossenPanel} und lässt dann
 * einen Kostentenor nach der Baumbach'schen Kostenformel erzeugen.
 * 
 * @TODO TODO Werden Werte geprüft?
 *
 * @author Peter Schuster (setrok)
 * @date 10.11.2018
 *
 */
public class StreitgenossenBaumbachAction extends BeschriebeneAktion {
	private final StreitgenossenTabellenBehandelnd panel;
	private static final long serialVersionUID = 8105321861085009090L;

	/**
	 * Konstruktor.
	 * 
	 * @param panel    {@link StreitgenossenPanel}
	 * @param mnemonic {@link Action#MNEMONIC_KEY}
	 */
	public StreitgenossenBaumbachAction(StreitgenossenTabellenBehandelnd panel, LangBeschriftung beschriftung) {
		super(beschriftung);
		this.panel = panel;
	}
	// String name, String shortDescription, , String actionCommandKey
	// super(name, shortDescription, mnemonic, actionCommandKey);
	// , int mnemonic, mnemonic, KeyEvent.VK_T

	@Override
	public void actionPerformed(ActionEvent ae) {
		/*
		 * Wenn Dwbk vorhanden, dann die Werte aller drei Tabellen zusammenfügen,
		 * ansonsten nur die von Klägern und Beklagten.
		 */
		if (!panel.pruefeTabellen()) {
			Throwable e = new IllegalArgumentException(StreitgenossenPanel.FEHLER_NIX_PARTEIEN_LANG);
			// StreitgenossenPanel.FEHLER_NIX_PARTEIEN,
			FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
			return;
		}
		try {
			final VerfahrensDatenContainer container = new VerfahrensDatenContainer2GUI().leseAlleWertefuerContainer();
			new TenorDialog(Kostenrechner.getInstance(), new BaumbachFormel(container).erzeugeContainer()).showDialog();
		} catch (IllegalArgumentException iae) {
			FehlerDialog.zeigeFehler(StreitgenossenPanel.FEHLER_BERECHNUNG + iae.getLocalizedMessage(), iae);
		}
	}// .allgemein, container.streitgenossen, container
}