/**
 * StreitgenossenBkAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.baumbach;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.interfaces.StreitgenossenTabellenBehandelnd;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.baumbach.BeteiligtenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.view.baumbach.BeteiligtenHinzufuegenDialog;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Die {@link Action} fügt, je nach Auswahl im Konstruktor
 * {@link #StreitgenossenBeteiligterNeuAction(StreitgenossenPanel, int, String, String, int, String)},
 * einen neuen Kläger hinzu, {@link KeyEvent#VK_K}, einen neuen (einzelnen)
 * Beklagten hinzu, {@link KeyEvent#VK_B}, oder einen neuen Drittwiderbeklagten
 * hinzu, {@link KeyEvent#VK_D}.
 *
 * @author Peter Schuster (setrok)
 * @date 10.11.2018
 *
 */
public class StreitgenossenBeteiligterNeuAction extends BeschriebeneAktion {
	private final StreitgenossenTabellenBehandelnd panel;
	private static final long serialVersionUID = -6505409262360418812L;
	private int typ;

	/**
	 * Konstruktor.
	 * 
	 * @param panel        {@link StreitgenossenPanel}
	 * @param typ          Welcher Beteiligtentyp soll hinzugefügt werden:
	 *                     {@link Beteiligter#KLAEGER}, {@link Beteiligter#BEKLAGTE}
	 *                     oder {@link Beteiligter#DRITTWIDERBEKLAGTE}
	 * @param beschriftung eine {@link LangBeschriftung} mit {@link Action#NAME},
	 *                     {@link Action#SHORT_DESCRIPTION},
	 *                     {@link Action#MNEMONIC_KEY} und
	 *                     {@link Action#ACTION_COMMAND_KEY}
	 */
	public StreitgenossenBeteiligterNeuAction(StreitgenossenTabellenBehandelnd panel, int typ,
			LangBeschriftung beschriftung) {
		super(beschriftung);
		this.panel = panel;
		this.typ = typ;
	}
	// super(name, shortDescription, mnemonic, mnemonic, actionCommandKey);
	// String name, String shortDescription,, String actionCommandKey, int mnemonic
	// , KeyEvent.VK_K, KeyEvent.VK_B, KeyEvent.VK_D

	@Override
	public void actionPerformed(ActionEvent ae) {
		beteiligtenHinzufuegen(panel.getTableModel(typ));
	}

	/**
	 * Die Methode beteiligtenHinzufuegen dient dazu, den JDialog
	 * BeteiligtenHinzufuegen zum Hinzufügen eines neuen BaumbaachBeteiligten
	 * auszulesen. Dazu liest sie BaumbachTableModel aus, sofern dieses Werte
	 * enthält, kopiert den letzten Eintrag und fügt den Rückgabewert wieder als
	 * neuen Eintrag der Tabelle hinzu.
	 * 
	 * @param btm das {@link BeteiligtenTableModel}, in das der Beteiligte eingefügt
	 *            werden soll.
	 */
	private void beteiligtenHinzufuegen(BeteiligtenTableModel btm) {
		BaumbachBeteiligter bt;
		if (btm.getRowCount() > 0)
			bt = btm.getRow(btm.getRowCount() - 1);
		else
			bt = new BaumbachBeteiligter(btm.getBeteiligtenTyp(), Beteiligter.WEIBLICH, 0.0, 0.0, false);
		BeteiligtenHinzufuegenDialog btd = new BeteiligtenHinzufuegenDialog(Kostenrechner.getInstance(), bt,
				panel.isWiderKlageAnzeigen());
		bt = btd.showDialog();
		/*
		 * Wenn versucht wird, mehr als einen Kläger zum btm hinzuzufügen, wirft dieses
		 * eine IllegalArgumentException
		 */
		if (bt != null)
			try {
				btm.addRow(bt);
			} catch (IllegalArgumentException e) {
				FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
			}
	}
}