/*
 * HtmlHelper.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.ConcurrentModificationException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;

import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Die Klasse kann den Inhalt einer HTML-Datei einlesen und ihn formatiert in
 * das übergebene {@link StyledDocument} schreiben sowie CSS-Dateien einlesen
 * und zuweisen.
 *
 * @author Peter Schuster (setrok)
 * @date 17 Apr 2019
 *
 */
public class HtmlHelper {
	/**
	 * Liest den Inhalt einer HTML-Datei ein und schreibt ihn formatiert in das
	 * übergebene {@link StyledDocument}
	 * 
	 * @param name der Name der HTML-Datei in den resources
	 * @param doc  das {@link StyledDocument} des JTextPanes, das gefüllt werden
	 *             soll
	 * @param kit  das {@link HTMLEditorKit} des JTextPanes, das gefüllt werden
	 *             soll.
	 */
	public void leseHtml(String name, StyledDocument doc, HTMLEditorKit kit) {
		InputStream is;
		try {
			is = getClass().getClassLoader().getResourceAsStream(name);
			doc.remove(0, doc.getLength());
			doc.putProperty("IgnoreCharsetDirective", new Boolean(true));
			kit.read(is, doc, 0);
		} catch (FileNotFoundException e) {
			FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
		} catch (IOException e) {
			FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
		} catch (BadLocationException e) {
			FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
		}
	}

	/**
	 * Die Methode dient dazu, eine CSS-Datei aus den Resources einzulesen und dem
	 * {@link HTMLEditorKit} zuzuweisen.
	 * 
	 * @param name den Namen der HTML-Datei in den
	 *             {@link ClassLoader#getResourceAsStream(String) Resources}.
	 * @param kit  das {@link HTMLEditorKit} des JTextPanes, das gefüllt werden
	 *             soll.
	 * 
	 * @url {@link https://stackoverflow.com/questions/17840914/how-to-provide-functionalty-for-changing-the-background-color-of-text-parts-in-h/18080249#18080249}
	 * @url {@link https://stackoverflow.com/questions/11177126/text-html-in-a-jtextpane-netbeans/11177448#11177448}
	 * @url {@link https://stackoverflow.com/questions/18448671/how-to-avoid-concurrentmodificationexception-while-removing-elements-from-arr}
	 *      - sonst {@link ConcurrentModificationException}
	 */
	public void leseCss(String name, HTMLEditorKit kit) {
		URL css = getClass().getClassLoader().getResource(name);
		StyleSheet sheet = new StyleSheet();
		Enumeration<?> e = kit.getStyleSheet().getStyleNames();
		List<String> names = new ArrayList<String>();
		// Erst sammeln
		while (e.hasMoreElements()) {
			String str = (String) e.nextElement();
			names.add(str);
		}
		// Danach kann ich erst entfernen, sonst ConcurrentModificationException
		Iterator<String> iter = names.iterator();
		while (iter.hasNext()) {
			String str = iter.next();
			kit.getStyleSheet().removeStyle(str);
		}
		sheet.importStyleSheet(css);
		kit.getStyleSheet().addStyleSheet(sheet);
	}

	/**
	 * Die Methode setzt in einer {@link JTextPane} das {@link HTMLEditorKit} und
	 * das {@link StyledDocument} und lädt die CSS.
	 * 
	 * @param textPane die {@link JTextPane}
	 */
	public void setzeKitUndDocument(JTextPane textPane) {
		HTMLEditorKit kit = new HTMLEditorKit();
		textPane.setEditorKit(kit);
		StyledDocument doc = (StyledDocument) kit.createDefaultDocument();
		textPane.setStyledDocument(doc);
//		helper.
		leseCss("resources/kostentenor.css", kit);
	}


	/*
	 * Die Methode dient dazu, eine HTML-Datei aus den Resources einzulesen.
	 * 
	 * @param name den Namen der HTML-Datei in den {@link
	 * ClassLoader#getResourceAsStream(String) Resources}.
	 * 
	 * @param doc public void htmlLesen(String name, StyledDocument doc,
	 * HTMLEditorKit kit) { InputStream is; try { is =
	 * getClass().getClassLoader().getResourceAsStream(name); doc.remove(0,
	 * doc.getLength()); doc.putProperty("IgnoreCharsetDirective", new
	 * Boolean(true)); kit.read(is, doc, 0); } catch (FileNotFoundException e) { new
	 * FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(),
	 * e).showDialog(); } catch (IOException e) { new
	 * FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(),
	 * e).showDialog(); } catch (BadLocationException e) { new
	 * FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(),
	 * e).showDialog(); } }
	 */
}