/**
 * SpeichereTenorAction.java
 * eu.gronos.kostenrechner.view.result (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.Action;
import javax.xml.bind.JAXBException;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.system.BooleanConfigOptionHorcher;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Die {@link Action} speichert den Tenortext im Rich Text Format (RTF) über
 * {@link RtfDocHelper#speichereRTF()}.
 * 
 * Wenn {@link Kostenrechner#debug} gesetzt ist (TODO
 * {@link BooleanConfigOptionHorcher} bauen TODO), wird über
 * {@link TenorXmlDatei} auch der {@link TenorDatenContainer} gespeichert.
 *
 * @author Peter Schuster (setrok)
 * @date 22 Nov 2018
 *
 */
public class SpeichereRtfAction extends BeschriebeneAktion/* AbstractAction */ {
	private final TenorDialog dialog;
	private TenorDatenContainer container;
	private static final long serialVersionUID = 1162624762983290295L;

	/**
	 * Konstruktor.
	 * 
	 * @param dialog           Verweis auf {@link TenorDialog}
	 * @param container        ein {@link TenorDatenContainer}
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 * @param acceleratorKey   {@link Action#ACCELERATOR_KEY}
	 */
	public SpeichereRtfAction(TenorDialog dialog, TenorDatenContainer container, LangBeschriftung beschriftung) {
		super(beschriftung);
		this.dialog = dialog;
		this.container = container;
	}
	// ,int mnemonic, KeyStroke acceleratorKey, mnemonic, acceleratorKey
	// String name,String shortDescription, String actionCommandKey,
	// name, shortDescription, actionCommandKey,

	@Override
	public void actionPerformed(ActionEvent e) {
		File datei = this.dialog.getDocHelper().speichereRTF();
		if (datei != null && dialog.isXmlSelected()) {
			datei = new File(datei.getAbsoluteFile() + TenorXmlDatei.DATEIENDUNG_GANZ);
			try {
				new TenorXmlDatei().speicherXml(container, datei);
			} catch (JAXBException je) {
				FehlerDialog.zeigeFehler("Fehler beim Umwandeln in XML", je);
			}
		}
	}
}