/**
 * TeilklageTenorAction.java
 * eu.gronos.kostenrechner.controller.gebuehren (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainer2GUI;
import eu.gronos.kostenrechner.logic.gebuehren.MehrkostenMethode;
import eu.gronos.kostenrechner.logic.gebuehren.QuotenMethode;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Der AbstractAction erzeugt den Kostentenor wahlweise nach der Quotenmethode
 * oder nach der Mehrkostenmethode.
 *
 * @author Peter Schuster (setrok)
 * @date 18 Feb 2019
 *
 */
public class TeilklageTenorAction extends BeschriebeneAktion {
	private static final long serialVersionUID = -172361077062445138L;
	private final TeilklageruecknahmePanel panel;

	public TeilklageTenorAction(TeilklageruecknahmePanel teilklageruecknahmePanel, LangBeschriftung beschriftung) {
		super(beschriftung);
		panel = teilklageruecknahmePanel;
	}
	// String name, String shortDescription,, String actionCommandKey,int
	// mnemonic,KeyEvent.VK_Q,
	// super(name, shortDescription, mnemonic, actionCommandKey);

	@Override
	public void actionPerformed(ActionEvent ae) {
		if (!panel.pruefeTabellen()) {
			panel.fehlerKeineGebuehren();
			return;
		}
		final VerfahrensDatenContainer verfahrensDaten = new VerfahrensDatenContainer2GUI()
				.leseAlleWertefuerContainer();
		TenorDatenContainer tenorDaten;
		try {
			if (panel.isQuotenMethodeVorziehen()) {
				tenorDaten = new QuotenMethode(verfahrensDaten).erzeugeContainer();
			} else {
				tenorDaten = new MehrkostenMethode(verfahrensDaten).erzeugeContainer();
			}
			new TenorDialog(panel.getKostenrechner(), tenorDaten).showDialog();
		} catch (IllegalArgumentException e) {
			FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
		}
	}// .allgemein, container.teilklageruecknahme)// .allgemein,
		// container.teilklageruecknahme), container
}
