/*
 * AllesBeschrifter.java
 * eu.gronos.kostenrechner.model (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.border.TitledBorder;

import eu.gronos.kostenrechner.model.Beschriftung;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.NameContainer;
import eu.gronos.kostenrechner.model.NameContainerMap;
import eu.gronos.kostenrechner.model.TastenKombination;

public class AllesBeschrifter {
	private JTabbedPane pane = null;

	public String beschreibeComponents(JDialog dialog) {
		NameContainerMap nc = new NameContainerMap();
		nc.setBeSchreibSet(buildFrom(dialog, 80000));
		return nc.toXmlString();
	}

	public String beschreibeComponents(JFrame frame) {
		NameContainerMap nc = new NameContainerMap();
		nc.setBeSchreibSet(buildFrom(frame, 80000));
		return nc.toXmlString();
	}

	public String beschreibeComponents(JPopupMenu pop) {
		NameContainerMap nc = new NameContainerMap();
		nc.setBeSchreibSet(buildFromM(pop, 80000));
		return nc.toXmlString();
	}

	public Set<NameContainer> buildFromM(JPopupMenu menu, int zaehler) {
		Set<NameContainer> set = new TreeSet<NameContainer>();
		set.addAll(buildFrom(zaehler, menu.getComponents()));
		return set;
	}

	public Set<NameContainer> buildFrom(JDialog dialog, int zaehler) {
		Set<NameContainer> set = new TreeSet<NameContainer>();
		set.add(buildFromComponent(dialog, zaehler));
		erhoeheZaehler(zaehler);
		set.addAll(buildFrom(zaehler, dialog.getContentPane().getComponents()));
		erhoeheZaehler(zaehler);
		set.addAll(buildFrom(zaehler, dialog.getLayeredPane().getComponents()));
		return set;
	}

	public Set<NameContainer> buildFrom(JPanel panel, int zaehler) {
		Set<NameContainer> set = new TreeSet<NameContainer>();
		set.addAll(buildFrom(zaehler, panel.getComponents()));
		return set;
	}

	public Set<NameContainer> buildFrom(JFrame frame, int zaehler) {
		Set<NameContainer> set = new TreeSet<NameContainer>();
		set.add(buildFromComponent(frame, zaehler));
		erhoeheZaehler(zaehler);
		set.addAll(buildFrom(zaehler, frame.getComponents()));
		erhoeheZaehler(zaehler);
		set.addAll(buildFrom(zaehler, frame.getLayeredPane().getComponents()));
		return set;
	}

	public NameContainer buildFromComponent(Component comp, int zaehler) {
		NameContainer cont = null;
		if (comp instanceof JTabbedPane) {
			pane = (JTabbedPane) comp;
		}
		if (comp instanceof AbstractButton) {
			cont = buildFrom(((AbstractButton) comp).getAction());
			if (((Beschriftung) cont).getTitle() == null || "".equals(((Beschriftung) cont).getTitle()))
				((Beschriftung) cont).setTitle(extractTitle((JComponent) comp));
			((Beschriftung) cont).setMnemonic(extractMnemonic((JComponent) comp));
		}
		if (comp instanceof JComponent) {
			if (cont == null)
				cont = new Beschriftung();
			((Beschriftung) cont).setTitle(extractTitle((JComponent) comp));
			((Beschriftung) cont).setShortDescription(extractDescription(comp, ((Beschriftung) cont).getTitle()));
			((Beschriftung) cont).setMnemonic(extractMnemonic((JComponent) comp));
		} else if (comp instanceof JDialog || comp instanceof JFrame) {
			if (cont == null)
				cont = new Beschriftung();
			// ((Beschriftung) cont).setTitle(((JDialog) comp).getTitle());
			((Beschriftung) cont).setTitle(extractTitle(comp));
			((Beschriftung) cont).setShortDescription(extractDescription(comp, ((Beschriftung) cont).getTitle()));
		} else {
			cont = new NameContainer();
		}
		try {
			Integer.valueOf(comp.getName());
			cont.setId(comp.getName());
		} catch (Exception e) {
			cont.setId(Integer.toString(zaehler));
		}
		return cont;
	}

	public LangBeschriftung buildFrom(Action action) {
		LangBeschriftung beschriftung = new LangBeschriftung();
		if (action == null)
			return beschriftung;
		String value = (String) action.getValue(Action.NAME);
		if (value != null)
			beschriftung.setTitle(value);
		value = (String) action.getValue(Action.SHORT_DESCRIPTION);
		if (value != null)
			beschriftung.setShortDescription(deHTML(value, beschriftung.getTitle()));
		value = (String) action.getValue(Action.LONG_DESCRIPTION);
		if (value != null)
			beschriftung.setLongDescription(value);
		value = (String) action.getValue(Action.ACTION_COMMAND_KEY);
		if (value != null)
			beschriftung.setActionCommandKey(value);
		Integer code = (Integer) action.getValue(Action.MNEMONIC_KEY);
		if (code != null)
			beschriftung.setMnemonic(code.intValue());
		KeyStroke acc = (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
		if (acc != null) {
			final TastenKombination kombi = new TastenKombination();
			kombi.setKeyCode(acc.getKeyCode());
			kombi.setAllModifiers(acc.getModifiers());
			beschriftung.setAcceleratorKey(kombi);
		}
		return beschriftung;
	}

	private Set<NameContainer> buildFrom(int zaehler, final Component[] components) {
		Set<NameContainer> set = new TreeSet<NameContainer>();
		for (JComponent comp : toList(components)) {
			if (comp instanceof JPanel || comp instanceof JMenuBar || comp instanceof JMenu
					|| comp instanceof JTabbedPane) {
				zaehler = erhoeheAuf(zaehler, 100);
			} else if (comp instanceof JScrollPane) {
				zaehler = erhoeheAuf(zaehler, 10);
			} else {
				zaehler = erhoeheZaehler(zaehler);
			}
			NameContainer cont = buildFromComponent(comp, zaehler);
			if (cont != null)
				set.add(cont);
		}
		return set;
	}

	private List<JComponent> toList(final Component[] components) {
		List<JComponent> comps = new ArrayList<JComponent>();
		if (components != null && components.length > 0) {
			for (Component comp : components) {
				if (comp instanceof JComponent) {
					comps.add((JComponent) comp);
					comps.addAll(toList(((JComponent) comp).getComponents()));
					if (comp instanceof JMenu) {
						comps.addAll(toList(((JMenu) comp).getPopupMenu().getComponents()));
					}
				}
			}
		}
		return comps;
	}

	private int erhoeheAuf(int zaehler, int teiler) {
		return zaehler + teiler - (zaehler % teiler);
	}

	private int erhoeheZaehler(int zaehler) {
		zaehler++;
		// && (zaehler % 10) != 0
		while ((zaehler % 10) != 1 && (zaehler % 10) != 5) {
			zaehler++;
		}
		return zaehler;
	}

	private String extractDescription(Component comp, String title) {
		if (comp == null)
			return "";
		String toolTipText = "";
		if (comp instanceof JPanel && pane != null && pane.indexOfComponent(comp) != -1)
			toolTipText = pane.getToolTipTextAt(pane.indexOfComponent(comp));
		else if (comp instanceof JDialog) {
			if (((JDialog) comp).getAccessibleContext() != null) {
				toolTipText = ((JDialog) comp).getAccessibleContext().getAccessibleDescription();
			}
		} else if (comp instanceof JFrame) {
			if (((JFrame) comp).getAccessibleContext() != null) {
				toolTipText = ((JFrame) comp).getAccessibleContext().getAccessibleDescription();
			}
		} else if (((JComponent) comp).getToolTipText() == null
				|| ((JComponent) comp).getToolTipText().trim().isEmpty()) {
			if (comp.getAccessibleContext() != null && comp.getAccessibleContext().getAccessibleDescription() != null
					&& !comp.getAccessibleContext().getAccessibleDescription().isEmpty())
				toolTipText = comp.getAccessibleContext().getAccessibleDescription();
			else
				return comp.toString();
		} else
			toolTipText = ((JComponent) comp).getToolTipText();
		if (title == null)
			title = "";
		if (toolTipText == null && comp.toString() != null)
			toolTipText = comp.toString();
		else if (toolTipText == null)
			toolTipText = "";
		return deHTML(toolTipText, title);
	}

	private String extractTitle(Component comp) {
		String title = "";
		if (comp instanceof JCheckBox) {
			title = ((JCheckBox) comp).getText();
		} else if (comp instanceof JLabel) {
			title = ((JLabel) comp).getText();
		} else if (comp instanceof AbstractButton) {
			title = ((AbstractButton) comp).getText();
		} else if (comp instanceof JPanel) {
			if (pane != null && pane.indexOfComponent(comp) != -1) {
				int index = pane.indexOfComponent(comp);
				title = pane.getTitleAt(index);
				System.out.printf("Index: %d, Title: %s", index, title);
			} else if (((JPanel) comp).getBorder() != null && ((JPanel) comp).getBorder() instanceof TitledBorder) {
				title = ((TitledBorder) ((JPanel) comp).getBorder()).getTitle();
			}
		} else if (comp.getAccessibleContext() != null && comp.getAccessibleContext().getAccessibleName() != null) {
			title = comp.getAccessibleContext().getAccessibleName();
		} else {
		}
		if (title == null || "".equals(title) || title.trim().isEmpty()) {
			title = comp.getClass().toString();
		}
		return title;
	}

	private int extractMnemonic(JComponent comp) {
		int mnemonic = 0;
		if (comp instanceof AbstractButton) {
			mnemonic = ((AbstractButton) comp).getMnemonic();
		} else if (comp instanceof JLabel) {
			mnemonic = ((JLabel) comp).getDisplayedMnemonic();
		} else if (comp instanceof JPanel) {
			if (pane != null && pane.indexOfComponent(comp) != -1
					&& pane.getMnemonicAt(pane.indexOfComponent(comp)) > 0) {
				mnemonic = pane.getMnemonicAt(pane.indexOfComponent(comp));
			}
		}
		return mnemonic;
	}

	private String deHTML(String str, String title) {
		if (title == null)
			title = "";
		return str.replace("<html>", "").replace("</html>", "").replace(String.format("<h3>%s</h3>", title), "")
				.replace(String.format("<h1>%s</h1>", title), "").replace("<p>", "").replace("</p>", "")
				.replace("<br>", "\n");
	}
}