/*
 * AkzeptanzDialog.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.GridBagConstraints;
import java.awt.HeadlessException;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;

import eu.gronos.kostenrechner.controller.files.HtmlHelper;
import eu.gronos.kostenrechner.model.Beschriftung;
import eu.gronos.kostenrechner.model.NameContainerSammlung;
import eu.gronos.kostenrechner.view.FontHelfer;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.result.FontHelferForWindow;

/**
 * Die Klasse ist ein {@link HinzufuegenDialog}, der nach der Zustimmung zur
 * Übersendung von Daten fragt.
 *
 * @author Peter Schuster (setrok)
 * @date 23 Apr 2019
 *
 */
public abstract class AkzeptanzDialog extends HinzufuegenDialog<Boolean> {

	private static final long serialVersionUID = -6845012067282320046L;
	private JTextPane textPane;
	private HTMLEditorKit kit;
	private StyledDocument doc;

	/**
	 * Konstruktor: erzeugt eine Instanz der {@link LizenzAkzeptanzDialog}.
	 * 
	 * @param owner ein {@link JFrame}
	 * @param title eine {@link Beschriftung} mit dem Titel usw.
	 * @throws HeadlessException
	 */
	public AkzeptanzDialog(JFrame owner, Beschriftung title) throws HeadlessException {
		super(owner, title);
	}

	/**
	 * @return {@link Boolean#TRUE} - wenn Ok gedrückt wird, ist alles gut.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Boolean baueRueckgabewert() {
		return Boolean.TRUE;
	}

	/**
	 * Die Methode setzt eine {@link JTextPane} mit {@link JScrollPane} ins Fenster
	 * und lädt HTML.
	 * 
	 * @param resource der Name der HTML-Datei in den resources
	 */
	protected void baueTextPane(String resource) {
		textPane = (JTextPane) NameContainerSammlung.BESCHRIFTUNGEN.get(73011).applyTo(new JTextPane());
		setKit(new HTMLEditorKit());
		textPane.setEditorKit(getKit());
		textPane.setEditable(false);
		setDoc((StyledDocument) getKit().createDefaultDocument());
		textPane.setStyledDocument(getDoc());
		HtmlHelper helper = new HtmlHelper();
		helper.leseHtml(resource, getDoc(), getKit());
		helper.leseCss("resources/kostentenor.css", kit);
		// Caret ganz nach oben setzen
		textPane.setCaretPosition(0);
		FontHelferForWindow fufHelfer = new FontHelferForWindow(this);
		fufHelfer.setPreferredSizeFromAttributeSet(getDoc().getParagraphElement(0).getAttributes(),
				FontHelfer.MAX_SPALTEN, FontHelfer.MAX_ZEILEN);
		JScrollPane scpLizenz = (JScrollPane) NameContainerSammlung.BESCHRIFTUNGEN.get(73010)
				.applyTo(new JScrollPane(textPane));
		textPane.addHyperlinkListener(HinzufuegenDialog.HYPERLINK_LISTENER);
		contentPanel.add(scpLizenz, GitterBeutelBeschraenkungen.getInstance(0, 0, 3, 1, GridBagConstraints.BOTH, true));
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	}

	/**
	 * Wenn der {@link HinzufuegenDialog#getOkAction() Ok-Button} gedrückt wird, ist
	 * alles gut.
	 * 
	 * @return <code>true</code>
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	public boolean parseEingabe() {
		return true;
	}

	/**
	 * @return gibt {@link #doc} als {@link StyledDocument} zurück.
	 */
	public StyledDocument getDoc() {
		return doc;
	}

	/**
	 * @param doc d. {@link #doc}, d. gesetzt werden soll als
	 *            {@link StyledDocument}.
	 */
	private void setDoc(StyledDocument doc) {
		this.doc = doc;
	}

	/**
	 * @return gibt {@link #kit} als {@link HTMLEditorKit} zurück.
	 */
	public HTMLEditorKit getKit() {
		return kit;
	}

	/**
	 * @param kit d. {@link #kit}, d. gesetzt werden soll als {@link HTMLEditorKit}.
	 */
	private void setKit(HTMLEditorKit kit) {
		this.kit = kit;
	}

}