/**
 * LizenzAkzeptanzHinzufuegen.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import eu.gronos.kostenrechner.controller.system.DatenschutzNoergler;
import eu.gronos.kostenrechner.model.Beschriftung;
import eu.gronos.kostenrechner.model.NameContainerSammlung;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.TabbedPaneFactory;

/**
 * Ein {@link HinzufuegenDialog}, der vor dem Absenden einer E-Mail die
 * Zustimmung zur Datenübertragung abwartet.
 *
 * @author Peter Schuster (setrok)
 * @date 22 Apr 2019
 *
 */
public class DatenschutzAkzeptanzDialog extends AkzeptanzDialog {

	private static final String RESOURCES_DSGVO_BASIS = "resources/dsgvo-basis.html";
	private static final String RESOURCES_DSGVO_ANHANG = "resources/dsgvo-anhang.html";
	private static final long serialVersionUID = 5974122103013004015L;
	private final String htmlfile;
	private JTabbedPane tabbedPane;

	/**
	 * Konstruktor: erzeugt eine Instanz des {@link DatenschutzAkzeptanzDialog}.
	 * 
	 * @param owner ein {@link JFrame}
	 * @param index entweder {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_BASIS}
	 *              oder {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_ANHANG}
	 * @throws HeadlessException
	 */
	public DatenschutzAkzeptanzDialog(JFrame owner, int index) throws HeadlessException {
		this(owner, true, index);
	}

	/**
	 * Konstruktor: erzeugt eine Instanz der {@link DatenschutzAkzeptanzDialog}.
	 * 
	 * @param owner            ein {@link JFrame}
	 * @param zustimmungNoetig wenn <code>true</code>, dann wird der OK-Button in
	 *                         "Ja, ich stimme zu!" umbenannt.
	 * @param index            entweder
	 *                         {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_BASIS}
	 *                         oder
	 *                         {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_ANHANG}
	 * @throws HeadlessException
	 */
	public DatenschutzAkzeptanzDialog(JFrame owner, boolean zustimmungNoetig, int index) throws HeadlessException {
		super(owner, (Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(74000));
		baueTabbedPane(index);
		if (zustimmungNoetig) {
			NameContainerSammlung.BESCHRIFTUNGEN.get(74051).applyTo(super.getOkButton());
		}
		if (DatenschutzNoergler.OPTION_DSGVO_EMAIL_ANHANG == index) {
			this.htmlfile = RESOURCES_DSGVO_ANHANG;
		} else {
			this.htmlfile = RESOURCES_DSGVO_BASIS;
		}
		super.setOkButtonToBeFocussed(true);
	}

	/**
	 * Die Methode initialisiert die allgemeinen {@link JComponent}s für den
	 * Inhaltsbereich des Dialogs und wird vom Konstruktor
	 * {@link HinzufuegenDialog#HinzufuegenDialog(JFrame, String)} selbsttätig
	 * aufgerufen.
	 * 
	 * Vorsicht! In dieser Implementierung werden nur eine {@link JTabbedPane} und
	 * eine {@link JPanel} gebaut; die {@link JPanel} (contentPanel) wird aber der
	 * {@link JTabbedPane} noch nicht hinzugefügt. Das erledigt
	 * {@link #baueTabbedPane(int)}. Grund: Reihenfolge des Aufrufs, denn der
	 * Konstruktor der Oberklasse ruft bereits die hiesige Methode auf; erst dann
	 * kann der Konstruktor der hiesigen Klasse den Index übergeben.
	 */
	@Override
	protected void baueContentPane() {
		GridBagLayout gbl_contentPanel = new GridBagLayout();

		tabbedPane = new JTabbedPane();
		getContentPane().add(tabbedPane, BorderLayout.CENTER);

		contentPanel.setBorder(BORDER);
		contentPanel.setLayout(gbl_contentPanel);
	}

	/**
	 * Die Methode krönt je nach index den ersten oder zweiten Reiter der
	 * {@link JTabbedPane} auf aktiv und bestückt nur diesen.
	 * 
	 * @param index entweder {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_BASIS}
	 *              oder {@link DatenschutzNoergler#OPTION_DSGVO_EMAIL_ANHANG}
	 * 
	 * @see javax.swing.JTabbedPane#setSelectedIndex(int)
	 */
	private void baueTabbedPane(int index) {
		int activeTab;
		if (DatenschutzNoergler.OPTION_DSGVO_EMAIL_ANHANG == index) {
			// Zweites Tab
			activeTab = 1;
			TabbedPaneFactory.intoTabbedPane(tabbedPane, new JPanel(),
					(Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(74100));
			tabbedPane.setEnabledAt(0, false);
			TabbedPaneFactory.intoTabbedPane(tabbedPane, contentPanel,
					(Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(74200));
			tabbedPane.setEnabledAt(1, true);
		} else {
			// Erstes Tab
			activeTab = 0;
			TabbedPaneFactory.intoTabbedPane(tabbedPane, contentPanel,
					(Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(74100));
			tabbedPane.setEnabledAt(0, true);
			TabbedPaneFactory.intoTabbedPane(tabbedPane, new JPanel(),
					(Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(74200));
			tabbedPane.setEnabledAt(1, false);
		}
		tabbedPane.setSelectedIndex(activeTab);
	}

	@Override
	protected void fuelleContentPane() {
		baueTextPane(htmlfile);
	}

	/**
	 * @return gibt {@link HinzufuegenDialog#okButton} als {@link AbstractButton}
	 *         zurück.
	 */
	public AbstractButton getButton() {
		return super.getOkButton();
	}

}
//private static final String TITLE = "Datenschutzerklärung";
//TITLE
// AbstractAction ok = super.getOkAction();
// ok.putValue(Action.NAME, "Ja, ich stimme zu!");
// ok.putValue(Action.MNEMONIC_KEY, KeyEvent.VK_J);
// ok.putValue(Action.SHORT_DESCRIPTION, "Der Datenschutzerklärung zustimmen");
// tabbedPane.addTab("Einfacher E-Mail-Versand", null);
// tabbedPane.addTab("E-Mail-Versand mit Zusätzen", null);
// tabbedPane.setComponentAt(activeTab, contentPanel);