/**
 * FehlerDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.io.IOException;
import java.util.logging.Level;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.HTMLDocument;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.HtmlHelper;
import eu.gronos.kostenrechner.controller.system.FehlerHelper;
import eu.gronos.kostenrechner.controller.system.SchreibeEmailFehler;
import eu.gronos.kostenrechner.model.Beschriftung;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.NameContainerSammlung;
import eu.gronos.kostenrechner.view.FontHelfer;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.RechnerhammerIcons;

/**
 * Eine Klasse, die Fehlermeldungen ausführlicher darstellt.
 *
 * @author Peter Schuster (setrok)
 * @date 26.08.2014
 *
 */
public class FehlerDialog extends HinzufuegenDialog<Boolean> {

	private static final long serialVersionUID = 306283812188663774L;
	private String meldung;
	private Throwable throwable;
	private final StackTraceElement stack;
	public static final String FEHLER_BLANKO = "Fehler";
	/**
	 * Das Info-Fenster soll maximal 7 Zeilen enthalten
	 */
	private static final int MAX_ZEILEN_TRACE = 11;
	/**
	 * Die eigentliche Fehlermeldung soll nur zwei Zeilen haben.
	 */
	private static final int FEHLER_ZEILEN = 2;

	private final AbstractAction emailSchreibenFehler;

	/**
	 * Der Konstruktor initialisiert den Dialog und gibt eine StackTrace aus.
	 * 
	 * @param meldung   die Fehlermeldung
	 * @param throwable das {@link Throwable} (Oberklasse von {@link Exception}),
	 *                  das geworfen wurde.
	 */
	private FehlerDialog(String meldung, Throwable throwable) {
		super(Kostenrechner.getInstance(), ((Beschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(72000)));
		this.meldung = meldung;
		this.throwable = throwable;
		super.setOkButtonToBeFocussed(true);
		this.stack = throwable.getStackTrace()[0];
		String klasse = stack.getClassName();
		String methode = stack.getMethodName();
		Kostenrechner.getLogger().logp(Level.WARNING, klasse, methode, meldung, throwable);
		emailSchreibenFehler = new SchreibeEmailFehler(
				(LangBeschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(72045), this.throwable);
	}

	/**
	 * Die Methode füllt das contentPane mit einem Icon-Label, einer JTextArea für
	 * die eigentliche Meldung und einer JTextArea im JScrollPane für den
	 * StackTrace.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		JPanel meldungPane = new JPanel();
		meldungPane.setLayout(new FlowLayout(FlowLayout.LEFT));
		Border schwarz = BorderFactory
				.createLineBorder(UIManager.getLookAndFeel().getDefaults().getColor("Button.shadow"));
		meldungPane.setBorder(BorderFactory.createTitledBorder(schwarz));
		NameContainerSammlung.BESCHRIFTUNGEN.get(72100).applyTo(meldungPane);

		JLabel lblIcon = new JLabel(RechnerhammerIcons.getRechnerhammerRot());
		JTextArea jtaMeldung = (JTextArea) NameContainerSammlung.BESCHRIFTUNGEN.get(72101)
				.applyTo(new JTextArea(meldung));
		jtaMeldung.setRows(FEHLER_ZEILEN);
		jtaMeldung.setEditable(false);
		jtaMeldung.setOpaque(false);

		final FehlerHelper feHelper = new FehlerHelper(throwable);
		JTextPane tpnStackTrace = (JTextPane) NameContainerSammlung.BESCHRIFTUNGEN.get(72201).applyTo(new JTextPane());
		final FontHelferForComponent fhfc = new FontHelferForComponent(tpnStackTrace);

		HtmlHelper helper = new HtmlHelper();
		helper.setzeKitUndDocument(tpnStackTrace);
		final HTMLDocument doc = (HTMLDocument) tpnStackTrace.getStyledDocument();
		fhfc.createBoldStyle(doc, fhfc.createBaseStyle(doc));

		try {
			feHelper.baueFehlerText(doc, throwable);
			// feHelper.writeHTML(doc, (HTMLEditorKit) tpnStackTrace.getEditorKit());
			throwable.printStackTrace();
		} catch (BadLocationException | IOException e) {
			System.err.println(e.getLocalizedMessage());
			e.printStackTrace();
		}
		fhfc.setPreferredSizeFromAttributeSet(doc.getStyle("base"), FontHelfer.MAX_SPALTEN, MAX_ZEILEN_TRACE);
		tpnStackTrace.setCaretPosition(0);
		tpnStackTrace.setForeground(Color.RED);
		tpnStackTrace.setEditable(false);
		JScrollPane scpStackTrace = (JScrollPane) NameContainerSammlung.BESCHRIFTUNGEN.get(72200)
				.applyTo(new JScrollPane(tpnStackTrace));
		scpStackTrace.setVisible(Kostenrechner.debug);

		meldungPane.add(lblIcon);
		meldungPane.add(jtaMeldung);
		contentPanel.add(meldungPane,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		contentPanel.add(scpStackTrace,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, true));

		getButtonPane().add(((BeschriebeneAktion) emailSchreibenFehler).toButton());
		addToRootInputMap(emailSchreibenFehler);
	}

	@Override
	public boolean parseEingabe() {
		return true;
	}

	@Override
	protected Boolean baueRueckgabewert() {
		return Boolean.TRUE;
	}

	/**
	 * Die Methode zeigt über {@link FehlerDialog} einen Fehlertext an.
	 * 
	 * @param meldung   den anzuzeigenden Fehlertext
	 * @param throwable das {@link Throwable} (Oberklasse von {@link Exception}),
	 *                  das geworfen wurde.
	 */
	public static void zeigeFehler(String meldung, Throwable throwable) {
		if (Kostenrechner.getInstance() == null || !Kostenrechner.getInstance().isVisible()) {
			if (Kostenrechner.getLogger() == null) {
				System.err.println(meldung);
				throwable.printStackTrace();
			} else {
				StackTraceElement stack = throwable.getStackTrace()[0];
				String klasse = stack.getClassName();
				String methode = stack.getMethodName();
				Kostenrechner.getLogger().logp(Level.SEVERE, klasse, methode, meldung, throwable);
			}
		} else
			new FehlerDialog(meldung, throwable).showDialog();
	}
}

// "Beim Autor beschweren","e-mail", , KeyEvent.VK_E
// "Öffnet das Standard-E-Mail-Programm,\n" + "um eine E-Mail an den
// Programmierer zu verfassen",
// new JButton(emailSchreibenFehler));
//final FontHelferForWindow fhfw = new FontHelferForWindow(this);
//StringBuffer sb = f_helper.getFehlerText();
//JTextArea jtaStackTrace = new JTextArea();
//jtaStackTrace.setText(sb.toString());

// fufHelfer.setPreferredSizeFromAttributeSet(doc.getStyle("base"),
// FontHelfer.MAX_SPALTEN, FontHelfer.MAX_ZEILEN// );
// jtaStackTrace.setPreferredSize(calculatePreferredSizeFrom
// , fhfc.getCharDimension(fhfc.getFontMetrics(attrib)),
// jtaStackTrace.getMaximumSize(),
// jtaStackTrace.getPreferredSize()));
//jtaStackTrace.setRows(7);

/*
 * Der Konstruktor initialisiert den Dialog und gibt eine StackTrace aus.
 * 
 * @param title den Titel des Dialogs
 * 
 * @param meldung die Fehlermeldung
 * 
 * @param throwable das {@link Throwable} (Oberklasse von {@link Exception}),
 * das geworfen wurde. private FehlerDialog(String title, String meldung,
 * Throwable throwable) { super(Kostenrechner.getInstance(), ((Beschriftung)
 * NameContainerSammlung.BESCHRIFTUNGEN.get(72000)).setTitle(title));
 * this.meldung = meldung; this.throwable = throwable;
 * super.setOkButtonToBeFocussed(true); this.stack =
 * throwable.getStackTrace()[0]; String klasse = stack.getClassName(); String
 * methode = stack.getMethodName();
 * Kostenrechner.getLogger().logp(Level.WARNING, klasse, methode, meldung,
 * throwable); emailSchreibenFehler = new SchreibeEmailFehler(
 * (LangBeschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(72045),
 * this.throwable); }
 */